import os
import json
import math

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np


# Old plotting routine, needs updating
def plot_runtime(calculated_results, unique_amt_cores, unique_amt_systems):
    fig = plt.figure()
    ax = fig.add_subplot(111)

    color_options = ["blue", "green", "red", "black", "yellow", "magenta"]

    x_position_shift = 0
    y_position_shift = -0.05

    colors = [color_options[i] for i in range(len(unique_amt_systems))]
    for i, amt_systems in enumerate(unique_amt_systems):

        cores = []

        mean_times_linear = []
        mean_times_multiprocessing = []
        std_linear = []
        std_mp = []

        for el in calculated_results:
            if el["systems"] == amt_systems:
                cores.append(el["cores"] + x_position_shift)

                mean_times_linear.append(el["mean_time_sequential"])
                std_linear.append(el["std_sequential"])

                mean_times_multiprocessing.append(el["mean_time_multiprocessing"])
                std_mp.append(el["std_multiprocessing"])

                # # add number of runs its based on
                # plt.text(el['cores'] + x_position_shift+0.01, el['mean_ratio']+y_position_shift, el['total_runs'])

        ax.errorbar(
            cores,
            mean_times_linear,
            std_linear,
            color=colors[i],
            linewidth=1,
            marker="P",
            label="{} systems linear run".format(amt_systems),
        )
        ax.errorbar(
            cores,
            mean_times_multiprocessing,
            std_mp,
            linewidth=1,
            color=colors[i],
            marker="d",
            label="{} systems MP run".format(amt_systems),
        )
        x_position_shift += 0.1

    ax.set_title(
        "Total time for sequential and MP for different amounts of systems on {}".format(
            name_testcase
        )
    )
    ax.set_xlabel("Amount of cores used")
    ax.set_ylabel("Total time taken (s)")

    ax.set_xlim(0, max(unique_amt_cores) + 20)
    ax.set_yscale("log")
    ax.grid()
    ax.legend()

    fig.savefig(
        os.path.join(img_dir, "total_time_scaling_{}.{}".format(name_testcase, "png"))
    )
    fig.savefig(
        os.path.join(img_dir, "total_time_scaling_{}.{}".format(name_testcase, "pdf"))
    )
    fig.savefig(
        os.path.join(img_dir, "total_time_scaling_{}.{}".format(name_testcase, "eps"))
    )

    plt.show()

def amdahl(f, n):
    return 1.0 / ((1 - f) + (f / n))

def plot_speedup_and_efficiency(result_json_filenames, plot_output_dir, name_testcase, machine):
    """
    Plotting routine to plot the speedup and efficiency of scaling

    Takes the name_testcase and hostname values of 
    the first json file to add some info to the plots.
    """

    # Setup figure
    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()

    # Go over all result jsons
    for i, jsonfile in enumerate(result_json_filenames):
        print("Processing {}".format(jsonfile))
        with open(jsonfile, "r") as f:
            result_data = json.loads(f.read())

            # if i==0:
            #     name_testcase = result_data['name_testcase']
            #     hostname = result_data['hostname']

            # Get linear data
            linear_data = result_data["linear"]
            linear_mean = np.mean(linear_data)
            linear_stdev = np.std(linear_data)

            # Get multiprocessing data
            cpus, speedups, efficiencies, stddev_speedups = [], [], [], []
            for amt_cpus in result_data["mp"]:

                # Get mp data
                mp_data = result_data["mp"][amt_cpus]
                mp_mean = np.mean(mp_data)
                mp_stdev = np.std(mp_data)

                # Calc
                amt_cpus = int(amt_cpus)
                speedup = linear_mean / mp_mean
                stddev_speedup = (
                    math.sqrt((linear_stdev / linear_mean) ** 2 + (mp_stdev / mp_mean) ** 2)
                    * speedup
                )
                efficiency = speedup / int(amt_cpus)

                # Add to lists
                cpus.append(amt_cpus)
                efficiencies.append(efficiency)
                speedups.append(speedup)
                stddev_speedups.append(stddev_speedup)

            # Plot
            ax1.errorbar(
                cpus,
                speedups,
                stddev_speedups,
                linestyle="None",
                marker="^",
                label="Speed up & efficiency of {} systems".format(
                    result_data["amt_systems"]
                ),
            )

            # Plot the efficiencies
            ax2.plot(cpus, efficiencies, alpha=0.5)

            # x_position_shift += 0.1

    #####################
    # Extra plots

    # 100 % scaling line
    ax1.plot([1, max(cpus)], [1, max(cpus)], '--', alpha=0.25, label='100% scaling')

    # Amdahls law fitting



    # Old stuff
    # Do Amdahls law fitting
    # cores = np.arange(1, 48, 0.1)
    # values_list = []
    # par_step = 0.005
    # par_vals = np.arange(.95, 1, par_step)

    # for par_val in par_vals:
    #     values = amdahl(par_val, cores)
    #     values_list.append(values)

    # for i, values in enumerate(values_list):
    #     ax1.plot(cores, values, label="par_val={}".format(par_vals[i]))

    #################################
    # Adding plot make up
    ax1.set_title(
        "Speed up ratio vs amount of cores for different amounts of systems on {}".format(
            machine
        )
    )

    ax1.set_xlabel("Amount of cores used")
    ax1.set_ylabel("Speed up ratio (time_linear/time_parallel)")

    ax1.grid()
    ax1.legend(loc=4)
    ax1.set_xscale("log")
    ax2.set_xscale("log")

    fig.savefig(os.path.join(plot_output_dir, "speedup_scaling_{}.{}".format(name_testcase, "png")))
    fig.savefig(os.path.join(plot_output_dir, "speedup_scaling_{}.{}".format(name_testcase, "pdf")))
    plt.show()

#################################
# Files
SCALING_RESULT_DIR = "scaling_results"
FILENAMES = [
    "david-Lenovo-IdeaPad-S340-14IWL_100_systems.json"
    # "astro2_2500_systems.json",
    # "astro2_3000_systems.json",
]
RESULT_JSONS = []
for filename in FILENAMES:
    RESULT_JSONS.append(os.path.join(os.path.abspath(SCALING_RESULT_DIR), filename))

plot_speedup_and_efficiency(RESULT_JSONS, SCALING_RESULT_DIR, "Example", "laptop_david")